use num::traits::Float;
use std::iter::FromIterator;

pub struct SquaringAMDemodulator<'a, NumType>
where
    NumType: Float,
{
    iterator: Box<dyn Iterator<Item = NumType> + 'a>,
}

impl<'a, NumType> SquaringAMDemodulator<'a, NumType>
where
    NumType: Float,
{
    pub fn from<I>(iterator1: I) -> SquaringAMDemodulator<'a, NumType>
    where
        I: Iterator<Item = NumType> + 'a,
    {
        SquaringAMDemodulator {
            iterator: Box::new(iterator1),
        }
    }
}

impl<'a, NumType> Iterator for SquaringAMDemodulator<'a, NumType>
where
    NumType: Float,
{
    type Item = NumType;

    fn next(&mut self) -> Option<Self::Item> {
        match self.iterator.next() {
            Some(x) => Some((x * x).sqrt()),
            None => None,
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_amdemod() {
        let test_data = vec![-1_f32, 2_f32, -3_f32, 4_f32, -5_f32];

        let demod = SquaringAMDemodulator::from(test_data.into_iter());

        let result_data = Vec::from_iter(demod);
        assert_eq!(result_data.len(), 5);
        assert_eq!(result_data, [1_f32, 2_f32, 3_f32, 4_f32, 5_f32]);
    }
}
